/*$Id: d_logic.cc,v 13.42 1999/05/25 08:19:08 al Exp $ -*- C++ -*-
 * logic model and device.
 * netlist syntax:
 * device:  mxxxx  out gnd vdd in1 in2 ... family gatetype
 * model:   .model mname LOGIC <args>
 */
#include "ap.h"
#include "d_logic.h"
/*--------------------------------------------------------------------------*/
//		DEV_LOGIC::DEV_LOGIC();
//		DEV_LOGIC::DEV_LOGIC(const DEV_LOGIC& p);
//	void	DEV_LOGIC::parse(CS& cmd);
// 	void	DEV_LOGIC::print(int,int)const;
//	void	DEV_LOGIC::expand();
//	double	DEV_LOGIC::probe_tr_num(const char *what)const;
//	double	DEV_LOGIC::probe_ac_num(const char *what)const;
// 	bool	DEV_LOGIC::dotr();
// 	void	DEV_LOGIC::trload();
// 	void	DEV_LOGIC::trunload();
// 	void	DEV_LOGIC::doac();
//	double	DEV_LOGIC::tr_review();
//	void	DEV_LOGIC::tr_advance();
/*--------------------------------------------------------------------------*/
extern NODE* nstat;
int DEV_LOGIC::Count = 0;
int LOGIC_COMMON::Count = -1;
static LOGIC_NONE Default_LOGIC(CC_STATIC);
/*--------------------------------------------------------------------------*/
DEV_LOGIC::DEV_LOGIC()
  :ELEMENT(),
   _lastchangenode(0),
   _quality(qGOOD),
   _failuremode("ok"),
   _oldgatemode(moUNKNOWN),
   _gatemode(moUNKNOWN)   
{
  attach_common(&Default_LOGIC);
  n = nodes;
  ++Count;
}
/*--------------------------------------------------------------------------*/
DEV_LOGIC::DEV_LOGIC(const DEV_LOGIC& p)
  :ELEMENT(p),
   _lastchangenode(0),
   _quality(qGOOD),
   _failuremode("ok"),
   _oldgatemode(moUNKNOWN),
   _gatemode(moUNKNOWN)   
{
  untested();
  for (int ii = 0;  ii <= PORTSPERGATE;  ii++){
    nodes[ii] = p.nodes[ii];
  }
  n = nodes;
  ++Count;
}
/*--------------------------------------------------------------------------*/
CARD& DEV_LOGIC::parse(CS& cmd)
{
  parselabel(cmd);
  int incount = parsenodes(cmd, PORTSPERGATE, 0, BEGIN_IN);
  if (incount <= BEGIN_IN){
    untested();
    incount = BEGIN_IN +1;
    cmd.warn(bDANGER);
  }
  incount -= BEGIN_IN;
  char modelname[LABELEN+1];
  cmd.ctostr(modelname, LABELEN, TOKENTERM);
  cmd.stuck();

  LOGIC_COMMON* c = 0;
  {if     (cmd.pmatch("AND" )) {c = new LOGIC_AND;}
  else if (cmd.pmatch("NAND")) {c = new LOGIC_NAND;}
  else if (cmd.pmatch("OR"  )) {c = new LOGIC_OR;}
  else if (cmd.pmatch("NOR" )) {c = new LOGIC_NOR;}
  else if (cmd.pmatch("XOR" )) {c = new LOGIC_XOR;}
  else if (cmd.pmatch("INV" )) {c = new LOGIC_INV;}
  else     {cmd.warn(bWARNING); c = new LOGIC_NONE;}
  }
  assert(c);
  c->incount = incount;
  c->set_modelname(modelname);
  attach_common(c);
  //probably (_gatemode == moUNKNOWN);
  return *this;
}
/*--------------------------------------------------------------------------*/
const CARD& DEV_LOGIC::print(OMSTREAM where, int)const
{
  const LOGIC_COMMON* c = dynamic_cast<const LOGIC_COMMON*>(common);
  assert(c);
  printlabel(where);
  printnodes(where, PORTSPERGATE);
  where << ' ' << c->modelname() << ' ' << c->name() << '\n'; 
  return *this;
}
/*--------------------------------------------------------------------------*/
CARD& DEV_LOGIC::expand()
{
  const LOGIC_COMMON* c = dynamic_cast<const LOGIC_COMMON*>(common);
  assert(c);
  const MODEL_LOGIC* m = dynamic_cast<const MODEL_LOGIC*>(attach_model());
  if (!m){
    untested();
    error(bERROR, "model %s is not a logic family (LOGIC)\n", c->modelname());
  }

  char cktname[BUFLEN];
  sprintf(cktname, "%s%s%u", c->modelname(), c->name(), c->incount);

  expandsubckt(cktname);

  {if (subckt().empty()){
    _gatemode = moDIGITAL;
    error(bDANGER, "%s: no model, forcing digital\n", printlabel());
  }else{
    _gatemode = OPT::mode;
    subckt().expand();
  }}

  assert(!constant()); /* is a BUG */
  return *this;
}
/*--------------------------------------------------------------------------*/
double DEV_LOGIC::probe_tr_num(const char *what)const
{
  untested();
  CS cmd(what);
  {if (cmd.pmatch("V")){
    untested();
    return n[OUTNODE].v0();
  }else { /* bad parameter */
    return NOT_VALID;
  }}
}
/*--------------------------------------------------------------------------*/
double DEV_LOGIC::probe_ac_num(const char *)const
{
  untested();
  return NOT_VALID;
}
/*--------------------------------------------------------------------------*/
void DEV_LOGIC::setgatemode()
{
  {if (subckt().exists()){
    assert(STATUS::iter[iSTEP] > 0);
    if (STATUS::iter[iSTEP] == 1){
      switch (SIM::phase){
      case pDC_SWEEP: untested();
      case pINIT_DC:
	_gatemode = (OPT::mode==moMIXED) ? moANALOG : OPT::mode;
	break;
      case pTRAN: /*nothing, set by review*/ break;
      case pNONE: unreachable();	     break;
      }
    }
  }else{
    assert(_gatemode == moDIGITAL);
  }}
  assert(_gatemode == moANALOG || _gatemode == moDIGITAL);
  assert(OPT::mode == moMIXED || OPT::mode == _gatemode 
	 || (subckt().empty() && _gatemode == moDIGITAL));

  NODE* outnode = &(nstat[n[OUTNODE].m]);
  outnode->set_mode(_gatemode);
}
/*--------------------------------------------------------------------------*/
bool DEV_LOGIC::dotr_analog()
{
  assert(_gatemode == moANALOG);
  assert(bypass() == false);
  set_converged(subckt().dotr());
  //            ^^^^^^^^^^^^^^^
  return converged();
}
/*--------------------------------------------------------------------------*/
bool DEV_LOGIC::dotr_digital()
{
  assert(_gatemode == moDIGITAL);
  NODE* outnode = &(nstat[n[OUTNODE].m]);
  
  switch (SIM::phase){
  case pDC_SWEEP: untested();
  case pINIT_DC:  
    tr_accept();
    setbypass(false); 
    //outnode->set_final_time(OVERDUE);
    break;
  case pTRAN:
    {if (SIM::time0 >= outnode->final_time()){
      outnode->propagate();
      setbypass(false);
    }else{
      setbypass(!(outnode->in_transit()) && SIM::bypass_ok);
    }}
    break;
  case pNONE:	  unreachable();		break;
  }
  
  {if (!bypass()){
    const LOGIC_COMMON* c = dynamic_cast<const LOGIC_COMMON*>(common);
    assert(c);
    const MODEL_LOGIC* m = dynamic_cast<const MODEL_LOGIC*>(c->model());
    assert(m);
    y0.x = 0.;
    y0.f1 = outnode->init_family(m).to_analog();
    y0.f0 = 0.;
    trace3("nobypass", y0.x, y0.f0, y0.f1);
    m0.x = 0.;
    m0.f1 = 1./m->rs;
    m0.c0 = y0.f1 / -m->rs;
    trace3("nobypass", m0.x, m0.c0, y0.f1);
    store_values();
    SIM::loadq.push(this);
  }else{
    trace3("bypass", y0.x, y0.f0, y0.f1);
    trace3("bypass", m0.x, m0.c0, y0.f1);
  }}
  set_converged(!outnode->is_unknown());
  return converged();
}
/*--------------------------------------------------------------------------*/
bool DEV_LOGIC::dotr()
{
  assert(!isbypassed());
  setgatemode();
  {if (_gatemode == moANALOG){
    return dotr_analog();
  }else{
    return dotr_digital();
  }}
}
/*--------------------------------------------------------------------------*/
CARD& DEV_LOGIC::trload()
{
  assert(_gatemode == moDIGITAL || _gatemode == moANALOG);
  {if (_gatemode == moANALOG){
    subckt().trload();
  }else{
    trload_passive();
  }}
  return *this;
}
/*--------------------------------------------------------------------------*/
CARD& DEV_LOGIC::trunload()
{
  untested();
  if (subckt().exists()){
    subckt().trunload();
  }
  return trunload_passive();
}
/*--------------------------------------------------------------------------*/
CARD& DEV_LOGIC::doac()
{
  {if (_gatemode == moANALOG){
    untested();
    subckt().doac();
  }else{
    untested();
    error(bWARNING, "%s: no logic in AC analysis\n", printlabel());
  }}
  return *this;
}
/*--------------------------------------------------------------------------*/
double DEV_LOGIC::tr_review()
{
  double nexttime = (_gatemode == moANALOG) ? subckt().tr_review() : NEVER;
  trace2(printlabel(), nexttime, (nexttime-SIM::time0));
  return nexttime;
}
/*--------------------------------------------------------------------------*/
/* tr_advance: the first to run on a new time step.
 * It sets up preconditions for the new time.
 */
CARD& DEV_LOGIC::tr_advance()
{
  if (_gatemode != _oldgatemode){
    switch (_oldgatemode){
    case moUNKNOWN: break;
    case moANALOG:  subckt().trunload(); break;
    case moDIGITAL: untested(); trunload_passive();  break;
    case moMIXED:   unreachable(); break;
    }
    NODE* outnode = &(nstat[n[OUTNODE].m]);
    outnode->set_mode(_gatemode);
    _oldgatemode = _gatemode;
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
/* tr_accept: This runs after everything has passed "review".
 * It sets up and queues transitions, and sometimes determines logic states.
 */
CARD& DEV_LOGIC::tr_accept()
{
  assert(_gatemode == moDIGITAL || _gatemode == moANALOG);
  const LOGIC_COMMON* c = dynamic_cast<const LOGIC_COMMON*>(common);
  assert(c);
  const MODEL_LOGIC* m = dynamic_cast<const MODEL_LOGIC*>(c->model());
  assert(m);
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  /* Check quality and get node info to local array. */
  /* side effect --- generate digital values for analog nodes */
  NODE* ns[PORTSPERGATE];
  {
    ns[OUTNODE] = m->tologic(n[OUTNODE]);
    _quality = ns[OUTNODE]->quality();  /* the worst quality on this device */
    _failuremode = ns[OUTNODE]->failure_mode();    /* what is wrong with it? */
    _lastchangenode = OUTNODE;		/* which node changed most recently */
    int lastchangeiter=ns[OUTNODE]->d_iter();/* iteration # when it changed */
    
    for (int ii = BEGIN_IN;  n[ii].e != INVALIDNODE;  ++ii){
      assert(ii < PORTSPERGATE);
      ns[ii] = m->tologic(n[ii]);
      if (ns[ii]->quality() < _quality){
	_quality = ns[ii]->quality();
	_failuremode = ns[ii]->failure_mode();
      }
      if (ns[ii]->d_iter() >= lastchangeiter){
	lastchangeiter = ns[ii]->d_iter();
	_lastchangenode = ii;
      }
    }
    /* If _lastchangenode == OUTNODE, no new changes, bypass may be ok.
     * Otherwise, an input changed.  Need to evaluate.
     * If all quality are good, can evaluate as digital.
     * Otherwise need to evaluate as analog.
     */
    trace0(printlabel());
    trace3(_failuremode, _lastchangenode, lastchangeiter, _quality);
  }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */  
  {if (want_analog()){
    if (_gatemode == moDIGITAL){
      error(bTRACE, "%s:%u:%g switch to analog, %s\n",
	    printlabel(), STATUS::iter[iTOTAL], SIM::time0, _failuremode);
      _oldgatemode = _gatemode;
      _gatemode = moANALOG;
    }
    assert(_gatemode == moANALOG);
    setbypass(false);
  }else{
    assert(want_digital());
    if (_gatemode == moANALOG){
      error(bTRACE, "%s:%u:%g switch to digital\n",
	    printlabel(), STATUS::iter[iTOTAL], SIM::time0);
      _oldgatemode = _gatemode;
      _gatemode = moDIGITAL;
    }
    assert(_gatemode == moDIGITAL);
    setbypass(SIM::bypass_ok && _lastchangenode==OUTNODE && SIM::phase==pTRAN);
    if (!bypass()){
      LOGICVAL newstate = c->logic_eval(&ns[BEGIN_IN]);
      //		     ^^^^^^^^^^
      {if (ns[OUTNODE]->is_unknown()){
	assert(SIM::phase == pINIT_DC  ||  SIM::phase == pDC_SWEEP);
	ns[OUTNODE]->force_initial_value(newstate);
	/* This happens when initial DC is digital.
	 * Answers could be wrong if order in netlist is reversed 
	 */
      }else{
	switch (newstate){
	case lvSTABLE0:	/*nothing*/		break;
	case lvRISING:  newstate=lvSTABLE0;	break;
	case lvFALLING: newstate=lvSTABLE1;	break;
	case lvSTABLE1:	/*nothing*/		break;
	case lvUNKNOWN: untested(); /*nothing*/	break;
	}
	if (newstate.lv_future() != ns[OUTNODE]->lv_future()){
	  ns[OUTNODE]->set_event(m->delay);
	  //assert(newstate == ns[OUTNODE]->lv_future());
	  if (_lastchangenode == OUTNODE){
	    untested();
	    error(bDANGER, "%s:%u:%g non-event state change\n",
		  printlabel(), STATUS::iter[iTOTAL], SIM::time0);
	  }
	}
      }}
    }
  }}
  return *this;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
