/*$Id: d_logic.h,v 13.42 1999/05/25 08:19:08 al Exp $ -*- C++ -*-
 * data structures and defaults for logic model.
 */
#ifndef D_LOGIC_H
#define D_LOGIC_H
#include "e_model.h"
#include "e_elemnt.h"
/*--------------------------------------------------------------------------*/
class MODEL_LOGIC : public MODEL_CARD {
friend class DEV_LOGIC;
public:
  explicit	MODEL_LOGIC();
		~MODEL_LOGIC()		{--Count;}
private: // overloaded virtuals
  CARD*		clone()const		{return new MODEL_LOGIC(*this);}
  CARD&		parse(CS&);
  const CARD&	print(OMSTREAM,int)const;
public:
  static int	count()			{return Count;}
private:
  explicit	MODEL_LOGIC(const MODEL_LOGIC& p):MODEL_CARD(p){unreachable();}
  NODE*		tologic(const node_t& node)const  
    {extern NODE* nstat; NODE& n = nstat[node.m];
    return &(n.init_family(this).to_logic());}
public:
			/* ----- digital mode ----- */
  double     delay;	/* propagation delay */
			/* -- conversion parameters both ways -- */
  double     vmax;	/* nominal volts for logic 1 */
  double     vmin;	/* nominal volts for logic 0 */
  double     unknown;	/* nominal volts for unknown (bogus) */
  double     range;	/* vmax - vmin */
  			/* ---- D to A conversion ---- */
  double     rise;	/* rise time (time in slope) */
  double     fall;	/* fall time (time in slope) */
  double     rs;	/* series resistance -- strong */
  double     rw;	/* series resistance -- weak */
  	        	/* ---- A to D conversion ---- */
  double     th1;	/* threshold for 1 as fraction of range */
  double     th0;	/* threshold for 0 as fraction of range */
  	        	/* ---- quality judgement parameters ---- */
  double     mr;	/* margin rise - how much worse rise can be */
  double     mf;	/* margin fall - how much worse fall can be */
  double     over;	/* overshoot limit - as fraction of range */
private:
  static int Count;
};
/*--------------------------------------------------------------------------*/
enum {PORTSPERGATE = 10};
/*--------------------------------------------------------------------------*/
class DEV_LOGIC : public ELEMENT {
public:
  explicit	DEV_LOGIC();
  explicit	DEV_LOGIC(const DEV_LOGIC& p);
		~DEV_LOGIC()		{--Count;}
private: // overloaded virtuals
  CARD*		clone()const	{untested(); return new DEV_LOGIC(*this);}
  CARD&		parse(CS&);
  const CARD&	print(OMSTREAM,int)const;
  CARD&		expand();
  bool		dotr();
  CARD&		trload();
  CARD&		trunload();
  CARD&		doac();
  double	tr_review();
  CARD&		tr_advance();
  CARD&		tr_accept();

  double	tr_involts()const	{untested(); return 0;}
  COMPLEX	ac_involts()const	{untested(); return 0;}
  double	probe_tr_num(const char*)const;
  double	probe_ac_num(const char*)const;

  bool		issubckt()const		{untested(); return true;}
  const char*	devtype()const
    {untested(); assert(common); return common->name();}
public:
  static int	count()			{return Count;}
private:
  void		setgatemode();
  bool		dotr_analog();
  bool		dotr_digital();
  bool		gate_eval(NODE**);
  bool		want_analog()const{return subckt().exists() &&
    ((OPT::mode == moANALOG) || (OPT::mode == moMIXED && _quality != qGOOD));}
  bool		want_digital()const{return subckt().empty() ||
    ((OPT::mode == moDIGITAL) || (OPT::mode == moMIXED && _quality == qGOOD));}
private:
  int		_lastchangenode;
  int		_quality;
  const char*	_failuremode;
  smode_t	_oldgatemode;
  smode_t	_gatemode;
  static int	Count;
  node_t	nodes[PORTSPERGATE+1];	/* PORTSPERGATE <= PORTSPERSUBCKT */
  enum {OUTNODE=0, GND_NODE=1, PWR_NODE=2, ENABLE=3, BEGIN_IN=4}; /* node labels */
};
/*--------------------------------------------------------------------------*/
class LOGIC_COMMON : public COMPONENT_COMMON {
protected:
  explicit	LOGIC_COMMON(int c=0)
    :COMPONENT_COMMON(c), incount(0) {++Count;}
  explicit	LOGIC_COMMON(const LOGIC_COMMON& p)
    :COMPONENT_COMMON(p), incount(p.incount) {untested(); ++Count;}
public:
		~LOGIC_COMMON()			{--Count;}
  static  int	count()				{return Count;}
  virtual LOGICVAL logic_eval(NODE**)const	= 0;
public:
  int		incount;
protected:
  static int	Count;
};
/*--------------------------------------------------------------------------*/
class LOGIC_AND : public LOGIC_COMMON {
public:
  explicit LOGIC_AND(int c=0)		  :LOGIC_COMMON(c) {untested();}
  explicit LOGIC_AND(const LOGIC_AND& p)  :LOGIC_COMMON(p){untested();++Count;}
  LOGICVAL logic_eval(NODE** ns)const {untested(); LOGICVAL out(lvSTABLE1);
    for (int ii=0; ii<incount; ++ii){out &= ns[ii]->lv();} return out;}
  virtual const char* name()const	  {return "and";}
};
/*--------------------------------------------------------------------------*/
class LOGIC_NAND : public LOGIC_COMMON {
public:
  explicit LOGIC_NAND(int c=0)		  :LOGIC_COMMON(c) {untested();}
  explicit LOGIC_NAND(const LOGIC_NAND& p):LOGIC_COMMON(p){untested();++Count;}
  LOGICVAL logic_eval(NODE** ns)const {untested(); LOGICVAL out(lvSTABLE1);
    for (int ii=0; ii<incount; ++ii){out &= ns[ii]->lv();} return ~out;}
  virtual const char* name()const	  {return "nand";}
};
/*--------------------------------------------------------------------------*/
class LOGIC_OR : public LOGIC_COMMON {
public:
  explicit LOGIC_OR(int c=0)		  :LOGIC_COMMON(c) {untested();}
  explicit LOGIC_OR(const LOGIC_OR& p)	  :LOGIC_COMMON(p){untested();++Count;}
  LOGICVAL logic_eval(NODE** ns)const {untested(); LOGICVAL out(lvSTABLE0);
    for (int ii=0; ii<incount; ++ii){out |= ns[ii]->lv();} return out;}
  virtual const char* name()const	  {return "or";}
};
/*--------------------------------------------------------------------------*/
class LOGIC_NOR : public LOGIC_COMMON {
public:
  explicit LOGIC_NOR(int c=0)		  :LOGIC_COMMON(c) {untested();}
  explicit LOGIC_NOR(const LOGIC_NOR& p)  :LOGIC_COMMON(p){untested();++Count;}
  LOGICVAL logic_eval(NODE** ns)const {untested(); LOGICVAL out(lvSTABLE0);
    for (int ii=0; ii<incount; ++ii){out |= ns[ii]->lv();} return ~out;}
  virtual const char* name()const	  {return "nor";}
};
/*--------------------------------------------------------------------------*/
class LOGIC_XOR : public LOGIC_COMMON {
public:
  explicit LOGIC_XOR(int c=0)		  :LOGIC_COMMON(c) {untested();}
  explicit LOGIC_XOR(const LOGIC_XOR& p)  :LOGIC_COMMON(p){untested();++Count;}
  LOGICVAL logic_eval(NODE** ns)const
    {untested(); return ns[0]->lv() ^ ns[1]->lv();}
  virtual const char* name()const	  {return "xor";}
};
/*--------------------------------------------------------------------------*/
class LOGIC_INV : public LOGIC_COMMON {
public:
  explicit LOGIC_INV(int c=0)		  :LOGIC_COMMON(c) {}
  explicit LOGIC_INV(const LOGIC_INV& p)  :LOGIC_COMMON(p){untested();++Count;}
  LOGICVAL logic_eval(NODE** ns)const	  {return ~ns[0]->lv();}
  virtual const char* name()const	  {return "inv";}
};
/*--------------------------------------------------------------------------*/
class LOGIC_NONE : public LOGIC_COMMON {
public:
  explicit LOGIC_NONE(int c=0)		  :LOGIC_COMMON(c) {}
  explicit LOGIC_NONE(const LOGIC_NONE& p):LOGIC_COMMON(p){untested();++Count;}
  LOGICVAL logic_eval(NODE**)const	  {untested(); return lvUNKNOWN;}
  virtual const char* name()const	  {return "error";}
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
