/*$Id: e_card.h,v 13.36 1999/05/07 08:37:41 al Exp $ -*- C++ -*-
 * branch structure type definitions
 * device types (enumerathon type?) 
 */
#include "e_node.h"
#include "e_cardlist.h"
#ifndef E_CARD_H
#define E_CARD_H
/*--------------------------------------------------------------------------*/
class CS;
class COMPONENT;
class COMPONENT_COMMON;
/*--------------------------------------------------------------------------*/
class CARD : public CKT_BASE {
private:
  int	    _evaliter;	// model eval iteration number
  CARD_LIST _subckt_list;
  CARD*	    _owner;
protected:
  char     label[LABELEN+1];
public: //insert, s__init, s__map
  node_t*  n;
private:
  node_t   _nodes[NODESPERBRANCH+1]; // nodes (0,1:out, 2,3:in)
  double   _val;	// value, for simple parts
  bool     _bypass;	// is bypassing now
  bool     _constant;	// eval stays the same every iteration
  //--------------------------------------------------------------------
public:   				// traversal functions
  CARD*	   find_in_scope(const char* name);
  //--------------------------------------------------------------------
public:					// virtuals. -- the important stuff
  virtual CARD*	      clone()const		= 0;
  virtual CARD&	      parse(CS&)		= 0;
  virtual const CARD& print(OMSTREAM,int)const	= 0;
  virtual CARD&	      expand()			= 0;
  virtual bool	      dotr()			= 0;
  virtual CARD&	      trload()			= 0;
  virtual CARD&	      trunload()		= 0;
  virtual CARD&	      doac()			= 0;
  virtual double      tr_review()		{return BIGBIG;}
  virtual CARD&	      tr_advance()		{return *this;}
  virtual CARD&	      tr_accept()		{return *this;}
  //--------------------------------------------------------------------
public:					// overloaded virtuals
  const char*	printlabel(OMSTREAM where=0)const;
  //--------------------------------------------------------------------
  //					// virtuals defined in base
  //virtual char*  probe_txt(const char*)const;	    // not overloaded here
  //virtual double probe_num(const char*)const;	    // shown here only
  //virtual double probe_tr_num(const char*)const;  // to complete the list
  //virtual double probe_ac_num(const char*)const;
  //virtual XPROBE probe_ac_ext(const char*)const;
  //--------------------------------------------------------------------
protected:				// create and destroy.
  explicit CARD();
  explicit CARD(const CARD&);
public:
  virtual  ~CARD()			{}
  //--------------------------------------------------------------------
public:					// query functions.
  static double	probe(const CARD*,const char*);
  int		connects_to(int node)const;
  //--------------------------------------------------------------------
protected:				// query functions. deferred inline
  bool	evaluated();
  //--------------------------------------------------------------------
public:					// query functions. virtual constant
  virtual bool	is1port()const		{return false;}
  virtual bool	is2port()const		{return false;}
  virtual bool	issource()const		{return false;}
  virtual bool	issubckt()const		{return false;}
  virtual bool	isdevice()const		{return false;}
  //--------------------------------------------------------------------
public:					// query functions. inline
  CARD_LIST&	   subckt()		{return _subckt_list;}
  const CARD_LIST& subckt()const	{return _subckt_list;}
  double	   value()const		{return _val;}
  CARD*		   owner()const		{return _owner;}
  int		   evaliter()const	{return _evaliter;}
  bool		   constant()const	{return _constant;}
  bool		   bypass()const	{return _bypass;}
  bool operator==(const char* n)const	{return strcasecmp(label,n)==0;}
  bool operator!=(const char* n)const	{return strcasecmp(label,n)!=0;}
  //--------------------------------------------------------------------
public:					// for debugging
  bool	isbypassed()const
    {return (owner() && (owner()->_bypass || owner()->isbypassed()));}
  //--------------------------------------------------------------------
public:					// modifiers.
  virtual CARD& setvalue(double v)	{_val = v; return *this;}
  virtual CARD& setvalue(double v, const COMPONENT_COMMON*)
					{untested(); return setvalue(v);}
  double*	setvalue()		{return &_val;}
  CARD& set_owner(CARD* o){assert(!_owner||_owner==o); _owner=o; return *this;}
  CARD& setconstant(bool c)		{_constant = c; return *this;}
  CARD& setbypass(bool b)		{_bypass = b;   return *this;}
  //--------------------------------------------------------------------
  // a very bad way to do parameter passing between the device and expression
 static double initial_voltage;
 static double initial_current;
 static double initial_condition;	/* could be either */ 
  //--------------------------------------------------------------------
  friend class CARD_LIST;
  friend CARD_LIST::fat_iterator findbranch(CS&,CARD_LIST::fat_iterator);
};
/*--------------------------------------------------------------------------*/
inline bool CARD::evaluated()
{
  if (_evaliter == STATUS::iter[iTOTAL]){
    return true;
  }else{
    _evaliter = STATUS::iter[iTOTAL];
    return false;
  }
}
/*--------------------------------------------------------------------------*/
inline CARD_LIST::fat_iterator findbranch(CS& cmd, CARD_LIST* cl){
  return findbranch(cmd, CARD_LIST::fat_iterator(cl));
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
