/*$Id: e_cardlist.cc,v 13.35 1999/05/04 08:52:43 al Exp $ -*- C++ -*-
 * Base class for "cards" in the circuit description file
 * This file contains functions that process a list of cards
 */
#include "l_stlextra.h"
#include "e_card.h" // includes e_cardlist.h
/*--------------------------------------------------------------------------*/
//	CARD_LIST*	     CARD_LIST::destroy();
//	CARD_LIST*	     CARD_LIST::expand();
//	bool		     CARD_LIST::dotr();
//	CARD_LIST&	     CARD_LIST::trload();
//	CARD_LIST&	     CARD_LIST::trunload();
//	CARD_LIST&	     CARD_LIST::doac();
//	double		     CARD_LIST::tr_review();
//	CARD_LIST&	     CARD_LIST::tr_advance();
//	CARD_LIST&	     CARD_LIST::tr_accept();
// CARD_LIST::iterator	     CARD_LIST::find(const char* short_name);
// CARD_LIST::const_iterator CARD_LIST::find(const char* short_name)const;
//	CARD*		     CARD_LIST::find_in_scope(const char* name);
//	CARD_LIST&	     CARD_LIST::erase(iterator ci);
/*--------------------------------------------------------------------------*/
CARD_LIST CARD_LIST::card_list;
CARD_LIST::fat_iterator CARD_LIST::putbefore(&card_list, card_list.end());
/*--------------------------------------------------------------------------*/
/* destroy: empty the list, destroy contents
 * contrast to detach_all
 * Beware: something else may be pointing to them, leaving dangling ptr.
 */
CARD_LIST& CARD_LIST::destroy()
{
  while (!cl.empty()){
    delete cl.back();
    cl.pop_back();
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
/* expand: expand (flatten) a list of components (subckts)
 * Scan component list.  Expand each subckt: create actual elements
 * for flat representation to use for simulation.
 * Recursive to allow for nested subckts.
 */
CARD_LIST& CARD_LIST::expand()
{
  for (iterator ci=begin(); ci!=end(); ++ci){
    (**ci).expand();
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
/* dotr: evaluate a list of models
 * evaluates a list (or sublist), checks convergence, etc.
 * does not load the matrix
 * argument is the head of the netlist.
 * recursively called to evaluate subcircuits
 */
bool CARD_LIST::dotr()
{
  bool isconverged = true;
  for (iterator ci=begin(); ci!=end(); ++ci){
    CARD* brh = *ci;
    if (!brh->constant() && !brh->evaluated()){
      isconverged &= brh->dotr();
    }
  }
  return isconverged;
}
/*--------------------------------------------------------------------------*/
/* trload: load list of models to the matrix
 * recursively called to load subcircuits
 */
CARD_LIST& CARD_LIST::trload()
{
  for (iterator ci=begin(); ci!=end(); ++ci){
    CARD* brh = *ci;
    if (!brh->constant() || !SIM::inc_mode){
      brh->trload();
    }else{
      untested();
    }
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
/* trunload: remove a list of models from the matrix
 * recursively called to unevaluate subcircuits
 */
CARD_LIST& CARD_LIST::trunload()
{
  for (iterator ci=begin(); ci!=end(); ++ci){
    (**ci).trunload();
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
CARD_LIST& CARD_LIST::doac()
{
  for (iterator ci=begin(); ci!=end(); ++ci){
    if (!(**ci).evaluated())
      (**ci).doac();
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
double CARD_LIST::tr_review()
{
  double worsttime = BIGBIG;
  for (iterator ci=begin(); ci!=end(); ++ci){
    double thistime = (**ci).tr_review();
    if (thistime < worsttime){
      worsttime = thistime;
    }
  }
  return worsttime;
}
/*--------------------------------------------------------------------------*/
/* tr_advance: first pass on a new time step
 */
CARD_LIST& CARD_LIST::tr_advance()
{
  for (iterator ci=begin(); ci!=end(); ++ci){
    (**ci).tr_advance();
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
/* tr_accept: final acceptance of a time step, before moving on
 */
CARD_LIST& CARD_LIST::tr_accept()
{
  for (iterator ci=begin(); ci!=end(); ++ci){
    (**ci).tr_accept();
  }
  return *this;
}
/*--------------------------------------------------------------------------*/
CARD_LIST::iterator CARD_LIST::find(const char* short_name)
{
  return find_ptr(begin(), end(), short_name);
}
/*--------------------------------------------------------------------------*/
CARD_LIST::const_iterator CARD_LIST::find(const char* short_name)const
{
  return find_ptr(begin(), end(), short_name);
}
/*--------------------------------------------------------------------------*/
CARD* CARD::find_in_scope(const char* name)
{
  CARD_LIST& cl = (owner()) ? owner()->subckt() : CARD_LIST::card_list;
  CARD_LIST::iterator ci = cl.find(name);
  if (ci == cl.end()){
    untested();
    error(bERROR, "can't find: %s\n", name);
    return NULL;
  }else{
    return *ci;
  }
}
/*--------------------------------------------------------------------------*/
CARD_LIST& CARD_LIST::erase(iterator ci)
{
  untested();
  assert(ci != end());
  delete *ci;
  cl.erase(ci);
  return *this;
}
/*--------------------------------------------------------------------------*/
CARD_LIST& CARD_LIST::erase(CARD* c)
{
  cl.remove(c);
  return *this;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
